//+------------------------------------------------------------------+
//| DLobotomy.mq4.mq4
//| Copyright  2007, Alexander Enchevich, Stephen "the Neuro Surgeon" Davis
//| http://www.metaquotes.net
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich, Stephen \"the Neuro Surgeon\" Davis"
#property link      "http://www.metaquotes.net"

#include <CommonLib.mqh>
#import "CommonLib.ex4"

#include <SeriesAnalysisLib.mqh>
#import "SeriesAnalysisLib.ex4"

#include <TradingLib.mqh>
#import "TradingLib.ex4"

#define _MagicNumber 200702031409
#define LOOKBACK	2
#define LOTS2TRADE	1

#define SLIPPAGE_ON_ENTER	3
#define SLIPPAGE_ON_CLOSE	4

#define TRADE_PERIOD	PERIOD_H1
#define TREND_PERIOD	PERIOD_D1

extern string MAMethods="0:Simple, 1:Exponential, 2:Smooth, 3:Linear Weighted";
extern string PriceCandleOptions="0:CLOSE, 1:OPEN, 2:HIGH, 3:LOW, 4:MEDIAN (H+L)/2, 5:TYPICAL (H+L+C)/3; WEIGHTED";
extern int g_iFastMAMethod=MODE_SMMA;
extern int g_iSlowMAMethod=MODE_SMA;
extern int g_iFastMAPriceMethod=PRICE_OPEN;
extern int g_iSlowMAPriceMethod=PRICE_OPEN;
extern int g_iFastMAPeriod=4;
extern int g_iSlowMAPeriod=4;

extern int g_iRSIPriceMethod=PRICE_CLOSE;
extern int g_iRSIPeriod=5;
extern double g_fRSILevelBuy=40.0;
extern double g_fRSILevelExitBuy=90.0;
extern double g_fRSILevelSell=60.0;
extern double g_fRSILevelExitSell=20.0;

string	g_strChartSymbol = "";
int 	g_iChartPeriod = -1;
bool 	g_bDisabled = false;
//+------------------------------------------------------------------+
double arrRSI	[LOOKBACK]; // holds RSI calculations for LOOKBACK ticks back

double arrRSILevelBuy[LOOKBACK];
double arrRSILevelExitBuy[LOOKBACK];
double arrRSILevelSell[LOOKBACK];
double arrRSILevelExitSell[LOOKBACK];

//+------------------------------------------------------------------+
int OpenAtMarket (  int iOrderType,
					double fLots,
					double fPrice,
					int iSlippage, 
					int iMagicNumber )
{
	if(iOrderType != OP_BUY && iOrderType != OP_SELL)
		return(-1);

// TODO : check for other limitations of OrderSend
// TODO : Implement context locking!
	if(fLots < 0.1)
		return(-1);

	string strOperation = OperationToStr( iOrderType );

	double fStopLoss = Ask-10000*Point;
	double fTakeProfit = Ask+999*Point;
	color iColor = RoyalBlue;
	
	if( OP_SELL == iOrderType )
	{		
		fStopLoss = Bid+10000*Point;
		fTakeProfit = Bid-999*Point;
		iColor = OrangeRed;
	}

	int iTicket = 
	OrderSend(	Symbol(), iOrderType, 
				NormalizeDouble(fLots, 1), NormalizeDouble(fPrice, 4),
				iSlippage, fStopLoss, fTakeProfit,
				strOperation, iMagicNumber, 0, iColor );

	if( -1 == iTicket )
	{
		Alert("OpenAtMarket: OrderSend Error ", GetLastError() );
	}
	
	return(iTicket);
}
//+------------------------------------------------------------------+
void CloseAllPositions( int iOrderType, 
						int iMagicNumber,
						double fPreferredClosePrice,
						int iSlippage, 
						color iColor )
{
	for( int iOrderPos=OrdersTotal(); iOrderPos>0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos-1, SELECT_BY_POS ) )
		{
			Alert ( "CloseAllPositions: Unable to select order at position ", 
					iOrderPos-1, " Error # ", GetLastError() );
		}
		else if(OrderSymbol() == Symbol() &&
				OrderMagicNumber() == iMagicNumber )
		{
			bool bCloseAll = (iOrderType==-1);

			if( bCloseAll || OrderType() == iOrderType )
			{
				int iTicket = OrderTicket();
				if( !OrderClose(iTicket, OrderLots(), fPreferredClosePrice, iSlippage, iColor) )
				{
					Alert ( "CloseAllPositions: Unable to Close Order #", iTicket,
							" at position ", iOrderPos-1,
							" Error # ", GetLastError() );
				}
			}
		}
	}
}
//+------------------------------------------------------------------+
void DrawTrendArrow(double iTrend)
{
	string strObjName = "Trend_"+
						TimeToStr(TimeCurrent(), TIME_DATE)+"_"+
						DoubleToStr( TimeHour(TimeCurrent()), 0 );

	if(ObjectFind(strObjName) == -1 )
	{
		ObjectCreate(strObjName, OBJ_ARROW, 0, TimeCurrent(), Ask+0.005, 0, 0, 0, 0);
		ObjectSet( strObjName, OBJPROP_ARROWCODE, SYMBOL_STOPSIGN );
		ObjectSet( strObjName, OBJPROP_COLOR, Yellow);
	}

	if(iTrend>0.0)
	{
		ObjectSet( strObjName, OBJPROP_ARROWCODE, SYMBOL_ARROWUP );
		ObjectSet( strObjName, OBJPROP_COLOR, Blue );
	}
	else if(iTrend<0.0)
	{
		ObjectSet( strObjName, OBJPROP_ARROWCODE, SYMBOL_ARROWDOWN );
		ObjectSet( strObjName, OBJPROP_COLOR, Red );
	}
	
}
//+------------------------------------------------------------------+
double TrendDirection()
{
	static int iOldTrend=0;
	int iGoBack = 0;
	double fSlowMA=iMA( g_strChartSymbol, TREND_PERIOD,
						g_iSlowMAPeriod, 0,
						g_iSlowMAMethod, g_iSlowMAPriceMethod,
						iGoBack );

	double fFastMA=iMA( g_strChartSymbol, TREND_PERIOD,
						g_iFastMAPeriod, 0,
						g_iFastMAMethod, g_iFastMAPriceMethod,
						iGoBack );

	int iNewTrend = Sign(fFastMA-fSlowMA);
	if(iNewTrend != iOldTrend )
	{
		Print ( "Trend changes from ", iOldTrend, " to ", iNewTrend );
		Print ( "MAs : Fast : ", fFastMA, " Slow : ",  fSlowMA );
	}

	iOldTrend = iNewTrend;
	return(fFastMA-fSlowMA);
}
//+------------------------------------------------------------------+
bool IsTrendUp()
{
	bool bUp = TrendDirection() > 0.0;

	DrawTrendArrow(TrendDirection());

	return(bUp);
}
//+------------------------------------------------------------------+
bool IsTrendDown()
{
	bool bDown = TrendDirection() < 0.0;

	DrawTrendArrow( TrendDirection() );
	return(bDown);
}
//+------------------------------------------------------------------+
bool IsCandleStart()
{
	static datetime iPeriodStartTime=0, iPeriodEndTime=0;
	static datetime iCurrTime = 0;
	static datetime iPrevTime = 0;

	iCurrTime = TimeCurrent();
	if( iCurrTime > iPeriodEndTime )
	{
		iPeriodStartTime = Time[0];
		iPeriodEndTime = Time[0]+Period()*60-1;
//		Print("@CandleStart: ", TimeToStr( iPeriodStartTime, TIME_SECONDS ) );

		iPrevTime = iCurrTime;
		return(true);
	}

	iPrevTime = iCurrTime;
	return(false);
}
//+------------------------------------------------------------------+
void CalcRSI()
{
	for( int i=LOOKBACK-1; i>0; i--)
		arrRSI[i] = arrRSI[i-1];

	arrRSI[0]=iRSI( g_strChartSymbol, g_iChartPeriod,
					g_iRSIPeriod,
					g_iRSIPriceMethod,
					1 );
}
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
	Print("BeginInit");
	ObjectCreate("Begin", OBJ_VLINE, 0, TimeCurrent(), 0, 0, 0, 0, 0);
	ObjectSet( "Begin", OBJPROP_COLOR, Lime );
	//----
	g_strChartSymbol=Symbol();
	if( g_strChartSymbol != "EURUSD" && g_strChartSymbol != "GBPUSD" )
	{
		g_bDisabled = true;
	}

	g_iChartPeriod=Period();
	if( !g_bDisabled ) // if chart Symbol was OK check Period
		if( g_iChartPeriod != TRADE_PERIOD )
		{
			g_bDisabled = true;
		}

	if( g_bDisabled )
	{
		MessageBox("Please only use this advisor with a EURUSD or GBPUSD 1-hr chart!",
					"Advisor Disabled!");
	}
	else
	{
		ArrayInitialize(arrRSILevelBuy,		g_fRSILevelBuy);
		ArrayInitialize(arrRSILevelExitBuy, g_fRSILevelExitBuy);
		ArrayInitialize(arrRSILevelSell,	g_fRSILevelSell);
		ArrayInitialize(arrRSILevelExitSell,g_fRSILevelExitSell);
		
		CalcRSI();
		
		Print("--------------------");
		Print(" FastMAMethod ", g_iFastMAMethod);
		Print(" SlowMAMethod ", g_iSlowMAMethod);
		Print(" FastMAPriceMethod ", g_iFastMAPriceMethod);
		Print(" SlowMAPriceMethod ", g_iSlowMAPriceMethod);
		Print(" FastMAPeriod ", g_iFastMAPeriod);
		Print(" SlowMAPeriod ", g_iSlowMAPeriod);

		Print(" RSIPriceMethod ", g_iRSIPriceMethod);
		Print(" RSIPeriod ", g_iRSIPeriod);
		Print(" RSILevelBuy ", g_fRSILevelBuy);
		Print(" RSILevelExitBuy ", g_fRSILevelExitBuy);
		Print(" RSILevelSell ", g_fRSILevelSell);
		Print(" RSILevelExitSell ", g_fRSILevelExitSell);
		Print("--------------------");
	}

	Print("End Init");
	return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
	//----

	//----
	return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
	int iTicket = -1;
	if( g_bDisabled ) return(0);

	CalcRSI();

	if( IsBuyPosition(iTicket, _MagicNumber) )
	{
		color colorCloseBuy = Black; // Black means "don't close".
		if( IsTrendDown())
		{
			Print("Closing Buy position(s) - trend changed!");
			colorCloseBuy = Red;
		}
		else if( IsCrossDown(0, LOOKBACK-1, arrRSI, arrRSILevelExitBuy) )
		{
			Print("Closing Buy position(s) for profit");
			colorCloseBuy = Green;
		}
		else if( IsCrossDown(0, LOOKBACK-1, arrRSI, arrRSILevelExitSell) )
		{
			Print("Closing Buy position(s) to cut losses");
			colorCloseBuy = Orange;
		}

		if( colorCloseBuy != Black )
		{
			CloseAllPositions ( OP_BUY,
								_MagicNumber, 
								Bid,
								SLIPPAGE_ON_CLOSE, 
								colorCloseBuy );
		}
	}

	if( IsSellPosition(iTicket, _MagicNumber) )
	{
		color colorCloseSell = Black; // Black means "don't close".
		if( IsTrendUp() )
		{
			Print("Closing Sell position(s) - trend changed!");
			colorCloseSell = Red;
		}
		else if( IsCrossUp(0, LOOKBACK-1, arrRSI, arrRSILevelExitSell) )
		{
			Print("Closing Sell position(s) for profit");
			colorCloseSell = Green;
		}
		else if( IsCrossUp(0, LOOKBACK-1, arrRSI, arrRSILevelExitBuy) )
		{
			Print("Closing Sell position(s) to cut losses ");
			colorCloseSell = Orange ;
		}
		
		if( colorCloseSell != Black )
		{
			CloseAllPositions ( OP_SELL,
								_MagicNumber,
								Ask,
								SLIPPAGE_ON_CLOSE,
								colorCloseSell );
		}
	}
//	else	
//	if( IsEmptyPosition(_MagicNumber) )
	{
		if( IsTrendUp() )
		{
			if( IsCrossUp(0, LOOKBACK-1, arrRSI, arrRSILevelBuy) )
			{
				iTicket = 
				OpenAtMarket (  OP_BUY, LOTS2TRADE,
								NormalizeDouble(Ask, 4), SLIPPAGE_ON_ENTER,
								_MagicNumber );
			
				if( -1 != iTicket )
				{
					Print("  arrRSI: ", ArrDToStr(arrRSI, 2) );
					Print("  Buy Level is @ :", ArrDToStr(arrRSILevelBuy, 4) );
					Print("OrderSend BUY returned ", iTicket );
				}
			}
		}
		else if(IsTrendDown() )
		{
			if( IsCrossDown(0, LOOKBACK-1, arrRSI, arrRSILevelSell) )
			{
				iTicket = 
				OpenAtMarket (  OP_SELL, LOTS2TRADE,
								NormalizeDouble(Bid, 4), SLIPPAGE_ON_ENTER,
								_MagicNumber );

				if( -1 != iTicket )
				{
					Print("  arrRSI: ", ArrDToStr(arrRSI, 2) );
					Print("  Sell Level is @ ", ArrDToStr(arrRSILevelSell, 4) );
					Print("OrderSend SELL returned ", iTicket );
				}
			}
		}
	}

	return(0);
}
//+------------------------------------------------------------------+

