//+------------------------------------------------------------------+
//|                                            SeriesAnalysisLib.mq4 |
//|                            Copyright  2007, Alexander Enchevich |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich"
#property link      ""
#property library

#include <CommonLib.mqh>
#import "CommonLib.ex4"

#define FLAG_STOP_ON_FIRST_UP_X	0001
#define FLAG_STOP_ON_FIRST_DN_X	0002
#define FLAG_RECORD_FIRST_UP_X	0004
#define FLAG_RECORD_FIRST_DN_X	0008
#define FLAG_DELTAS_SIGNS_ONLY	0016 

#define CROSSBONUSVALUE 2

//+------------------------------------------------------------------+
void SanityCheckSeriesIndex(	int& iTickBackIndex,
						int iSizeFast,
						int iSizeSlow)
{
// Sanity checks, BEGIN
	if( iTickBackIndex < 0)
		iTickBackIndex= -iTickBackIndex;

	if( iTickBackIndex>= iSizeFast )
		iTickBackIndex= iSizeFast-1;
	if( iTickBackIndex>= iSizeSlow )
		iTickBackIndex= iSizeSlow-1;
// Sanity checks, END
}
//+------------------------------------------------------------------+
double CalcSeriesDeltas(int iTickBackStart,
						int iTickBackEnd, 
						double arrFastSeries[], 
						double arrSlowSeries[],
						double& arrDeltas[] )
{
	int iSizeFast = ArrayRange(arrFastSeries, 0);
	int iSizeSlow = ArrayRange(arrSlowSeries, 0);

	SanityCheckSeriesIndex( iTickBackStart, iSizeFast, iSizeSlow);
	SanityCheckSeriesIndex( iTickBackEnd, iSizeFast, iSizeSlow);

	if( ArrayResize(arrDeltas, iTickBackEnd - iTickBackStart + 1 ) != 
		iTickBackEnd - iTickBackStart + 1 )
	{
		Print("Unable to resize Deltas Array");
		return (0.0);
	}
	
	double fAbsDeltaSum=0.0;
	for(int i=iTickBackStart; i<=iTickBackEnd; i++)
	{
		arrDeltas[i] = arrFastSeries[i] - arrSlowSeries[i];
		fAbsDeltaSum += MathAbs(arrDeltas[i]);
	}

	return (fAbsDeltaSum);
}
//+------------------------------------------------------------------+
// 
int PrivateCountCrosses(int& iNumCrossesUp,
						int& iNumCrossesDown,
						int& iFirstCrossUpIndex,
						int& iFirstCrossDownIndex,
						int iTickBackStart,
						int iTickBackEnd,
						double arrFastSeries[], 
						double arrSlowSeries[],
						int iFLAGS )
{
	int iSizeFast = ArrayRange(arrFastSeries, 0);
	int iSizeSlow = ArrayRange(arrSlowSeries, 0);

	SanityCheckSeriesIndex( iTickBackStart, iSizeFast, iSizeSlow);
	SanityCheckSeriesIndex( iTickBackEnd, iSizeFast, iSizeSlow);

	int iNumCrosses = 0;

	bool bWantFirstUp	=iFLAGS&FLAG_RECORD_FIRST_UP_X;
	bool bWantFirstDown	=iFLAGS&FLAG_RECORD_FIRST_DN_X;

	iFirstCrossUpIndex = -1;
	iFirstCrossDownIndex = -1;
	
	int iSign;
	int iPrevSign = Sign(arrFastSeries[iTickBackStart] - arrSlowSeries[iTickBackStart]);
//	Print(".  Sign[", iTickBackStart, "] = ", iPrevSign);

	for(int i=iTickBackStart+1; i<=iTickBackEnd; i++)
	{
		iSign = Sign(arrFastSeries[i] - arrSlowSeries[i]);
//		Print("   Sign[", i, "] = ", iSign);

		// has there been changes of sign in the MA series. Just touching
		// the other series (i.e. sign changed to 0 ) does not count.
		if(iSign != iPrevSign && iSign != 0 && iPrevSign != 0 )
		{
			iNumCrosses++;
			if( iSign < 0 )
			{
				iNumCrossesUp++;
				if( bWantFirstUp )
				{
					iFirstCrossUpIndex = i;
					bWantFirstUp = false;
					if((iFLAGS&FLAG_STOP_ON_FIRST_UP_X) != 0)
						break;
				}
			}
			else
			{
				iNumCrossesDown++;
				if( bWantFirstDown )
				{
					iFirstCrossDownIndex = i;
					bWantFirstDown = false;
					if((iFLAGS&FLAG_STOP_ON_FIRST_DN_X) != 0)
						break;
				}
			}
		}

		// record change in sign. 0 is undecisive so don't record.
		if( iSign != 0)
			iPrevSign = iSign;
	}

	return(iNumCrosses);
}
//+------------------------------------------------------------------+
int CountNumCrosses(int& iNumCrossesUp,
					int& iNumCrossesDown,
					int iTickBackStart,
					int iTickBackEnd, 
					double arrFastSeries[], 
					double arrSlowSeries[] )
{
	int iFirstCrossUpIndex;
	int iFirstCrossDownIndex;
	int iNumCrosses = PrivateCountCrosses(	iNumCrossesUp,
											iNumCrossesDown,
											iFirstCrossUpIndex,
											iFirstCrossDownIndex,
											iTickBackStart,
											iTickBackEnd, 
											arrFastSeries,
											arrSlowSeries,
											0 );
	return (iNumCrosses);
}
//+------------------------------------------------------------------+
int GetFirstCrossIndex( int iTickBackStart,
						double arrFastSeries[], 
						double arrSlowSeries[] )
{
	int iSizeFast = ArrayRange(arrFastSeries, 0);
	int iSizeSlow = ArrayRange(arrSlowSeries , 0);
	int iTickBackEnd = MathMin(iSizeFast, iSizeFast)-1;

	int iFirstCrossUpIndex, iNumCrossesUp=0;
	int iFirstCrossDownIndex, iNumCrossesDown=0;

	PrivateCountCrosses(iNumCrossesUp,
						iNumCrossesDown,
						iFirstCrossUpIndex,
						iFirstCrossDownIndex,
						iTickBackStart,
						iTickBackEnd, 
						arrFastSeries,
						arrSlowSeries,
						FLAG_STOP_ON_FIRST_UP_X|
						FLAG_STOP_ON_FIRST_DN_X|
						FLAG_RECORD_FIRST_UP_X|
						FLAG_RECORD_FIRST_DN_X );

	if( iFirstCrossUpIndex == -1 )
		return (iFirstCrossDownIndex);
	else
		return (iFirstCrossUpIndex);
}
//+------------------------------------------------------------------+
int GetFirstCrossUpIndex(	int iTickBackStart,
							double arrFastSeries[], 
							double arrSlowSeries[] )
{
	int iSizeFast = ArrayRange(arrFastSeries, 0);
	int iSizeSlow = ArrayRange(arrSlowSeries , 0);
	int iTickBackEnd = MathMin(iSizeFast, iSizeFast)-1;

	int iFirstCrossUpIndex, iNumCrossesUp;
	int iFirstCrossDownIndex, iNumCrossesDown;

	PrivateCountCrosses(iNumCrossesUp,
						iNumCrossesDown,
						iFirstCrossUpIndex,
						iFirstCrossDownIndex,
						iTickBackStart,
						iTickBackEnd, 
						arrFastSeries,
						arrSlowSeries,
						FLAG_STOP_ON_FIRST_UP_X|
						FLAG_RECORD_FIRST_UP_X );

	return (iFirstCrossUpIndex);
}
//+------------------------------------------------------------------+
int GetFirstCrossDownIndex(	int iTickBackStart,
							double arrFastSeries[], 
							double arrSlowSeries[] )
{
	int iSizeFast = ArrayRange(arrFastSeries, 0);
	int iSizeSlow = ArrayRange(arrSlowSeries , 0);
	int iTickBackEnd = MathMin(iSizeFast, iSizeFast)-1;

	int iFirstCrossUpIndex, iNumCrossesUp;
	int iFirstCrossDownIndex, iNumCrossesDown;

	PrivateCountCrosses(iNumCrossesUp,
						iNumCrossesDown,
						iFirstCrossUpIndex,
						iFirstCrossDownIndex,
						iTickBackStart,
						iTickBackEnd, 
						arrFastSeries,
						arrSlowSeries,
						FLAG_STOP_ON_FIRST_DN_X|
						FLAG_RECORD_FIRST_DN_X );

	return (iFirstCrossDownIndex);
}
//+------------------------------------------------------------------+
double  SeriesConvergence (	int iTickBackStart,
							int iTickBackEnd, 
							double arrFastSeries[], 
							double arrSlowSeries[] )
{
	int iFirstCrossUpIndex, iNumCrossesUp;
	int iFirstCrossDownIndex, iNumCrossesDown;

	int iNumCrosses = PrivateCountCrosses(	iNumCrossesUp,
											iNumCrossesDown,
											iFirstCrossUpIndex,
											iFirstCrossDownIndex,
											iTickBackStart,
											iTickBackEnd, 
											arrFastSeries,
											arrSlowSeries,
											FLAG_RECORD_FIRST_UP_X|
											FLAG_RECORD_FIRST_DN_X );

	// we are ok with a cross at the end of the series (close to sample 0)
	double fCrossBonus = 0.0;
	if(iNumCrosses > 1)
	{
		Print("iNumCrosses > 1");
		return (0);
	}
	else if(iNumCrosses==1)
	{
		Print("iNumCrosses==1");
		if(iNumCrossesUp==1)
		{
			// The Fast series crossed the Slow from down upwards.
			// a cross at 1 or 2 is a good indicator of convergence. We
			// can never actually have a cross at zero, because that's the
			// first value in the series (i.e. the one currently forming) 
			// and at this time we don't have yet enough data to estimate 
			// whether it is a cross. But we put 0 here anyway  :)
			if( iFirstCrossUpIndex >= 0 &&
				iFirstCrossUpIndex <  2 )
				fCrossBonus = CROSSBONUSVALUE;
			// a cross which occured some time ago is a sign of divergence
			else if(iFirstCrossUpIndex == iTickBackEnd ||
					iFirstCrossUpIndex == iTickBackEnd-1 )
				fCrossBonus = -CROSSBONUSVALUE;

			Print( "Cross UP, Bonus=", fCrossBonus, "; idx ", iFirstCrossUpIndex );
		}
		else
		{
			// same logic as for CrossUp above, except now the Fast 
			// series is crossing the slow series from up downwards
			if( iFirstCrossDownIndex >= 0 || 
				iFirstCrossDownIndex <= 2 )
				fCrossBonus = CROSSBONUSVALUE;
			else if(iFirstCrossDownIndex == iTickBackEnd ||
					iFirstCrossDownIndex == iTickBackEnd-1 )
				fCrossBonus = -CROSSBONUSVALUE;
			
			Print( "Cross DN, Bonus=", fCrossBonus, "; idx ", iFirstCrossDownIndex );
		}
	}
	
	double arrDeltas[];
	
	CalcSeriesDeltas (	iTickBackStart,
						iTickBackEnd, 
						arrFastSeries,
						arrSlowSeries,
						arrDeltas );
	
	double fTotalConvergence=0.0;
	double fTotalDeviations=0.0;
	int iSizeArrDeltas = ArrayRange(arrDeltas, 0)-1;
	for(int i=0; i<iSizeArrDeltas-1; i++)
	{
		double fDev = arrDeltas[i+1] - arrDeltas[i];
		fTotalConvergence += fDev;
		fTotalDeviations += MathAbs(fDev);
	}

	double fAvgDeviation=0.0;
	if(iSizeArrDeltas>1)
		fAvgDeviation = fTotalDeviations/(iSizeArrDeltas-1);

	if( -0.000001 < fAvgDeviation && fAvgDeviation < 0.000001 )
		return (0);
	
	fTotalConvergence += fCrossBonus*fAvgDeviation;

	return (fTotalConvergence/fAvgDeviation);
}
//+------------------------------------------------------------------+
bool IsCrossUp(	int iTickBackStart,
				int iTickBackEnd, 
				double arrFastSeries[], 
				double arrSlowSeries[] )
{
	int iNumCrossesUp=0;
	int iNumCrossesDown=0;
	int iFirstCrossUpIndex=-1;
	int iFirstCrossDownIndex=-1;

	if( PrivateCountCrosses(iNumCrossesUp,
							iNumCrossesDown,
							iFirstCrossUpIndex,
							iFirstCrossDownIndex,
							iTickBackStart,
							iTickBackEnd, 
							arrFastSeries, 
							arrSlowSeries,
							FLAG_RECORD_FIRST_UP_X|
							FLAG_RECORD_FIRST_DN_X )
			== 0 )
	{
		return(false);
	}

	if( iNumCrossesUp > 0 )
	{
		if( iFirstCrossDownIndex == -1 )
			return (true);
		else
			return (iFirstCrossDownIndex > iFirstCrossUpIndex);
	}

	return(false);
}
//+------------------------------------------------------------------+
bool IsCrossDown(int iTickBackStart,
				int iTickBackEnd, 
				double arrFastSeries[], 
				double arrSlowSeries[] )
{
	int iNumCrossesUp=0;
	int iNumCrossesDown=0;
	int iFirstCrossUpIndex=-1;
	int iFirstCrossDownIndex=-1;

	if( PrivateCountCrosses(iNumCrossesUp,
							iNumCrossesDown,
							iFirstCrossUpIndex,
							iFirstCrossDownIndex,
							iTickBackStart,
							iTickBackEnd, 
							arrFastSeries, 
							arrSlowSeries,
							FLAG_RECORD_FIRST_UP_X|
							FLAG_RECORD_FIRST_DN_X )
			> 0 )
	{
		if( iNumCrossesDown > 0 )
		{
			if( iFirstCrossUpIndex == -1 )
				return (true);
			else
				return (iFirstCrossUpIndex > iFirstCrossDownIndex);
		}
	}

	return(false);
}
//+------------------------------------------------------------------+
double SeriesAvgDelta(	int iTickBackStart,
						int iTickBackEnd, 
						double arrFastSeries[], 
						double arrSlowSeries[] )
{
	double arrDeltas[];
	double fAbsDeltaSum = CalcSeriesDeltas (iTickBackStart,
											iTickBackEnd, 
											arrFastSeries,
											arrSlowSeries,
											arrDeltas );

	int iNumDeltas = iTickBackEnd-iTickBackStart+1;
	if( iNumDeltas == 0 )
		return (0.0);

	return( fAbsDeltaSum / iNumDeltas );
}