//+------------------------------------------------------------------+
//|                                                   TradingLib.mq4 |
//|                            Copyright  2007, Alexander Enchevich |
//|                                                                  |
//| Note: Depends on CommonLib.                                      |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich"
#property link      ""
#property library

#include <CommonLib.mqh>
#import "CommonLib.ex4"

//+------------------------------------------------------------------+
double LotsOptimized(double fUseableMargin)
{
	//---- select lot size
	double fLotSize=NormalizeDouble(AccountFreeMargin()*fUseableMargin/1000.0,1);
	//---- calculate number of continuous losing orders
	int    iLosses=0;				// number of continuous losing orders
	int    orders=HistoryTotal();	// history orders total

	for(int i=orders-1;i>=0;i--)
	{
		if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
		{
			Alert("LotsOptimized: Error in history!"); 
			break; 
		}

		if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
		//----
		if(OrderProfit()>0) break;
		if(OrderProfit()<0) iLosses++;
	}

	if(iLosses>2) fLotSize=NormalizeDouble(fLotSize/(iLosses),1);
	//---- return lot size
	if(fLotSize<0.1) fLotSize=0.1;

	return(fLotSize);
}
//+------------------------------------------------------------------+
bool IsEmptyPosition(int iMagicNumber)
{
	for(int iOrderPos=0; iOrderPos<OrdersTotal(); iOrderPos++)
	{
		if( !OrderSelect( iOrderPos, SELECT_BY_POS ) )
		{
			Alert ( "IsEmptyPosition: Unable to select order at position ",
					iOrderPos, " Error # ", GetLastError() );
			continue;
		}

		if( OrderSymbol() == Symbol() && OrderMagicNumber() == iMagicNumber )
		{
			if( OrderType() == OP_BUY || OrderType() == OP_SELL )
				return(false);
	
			Print("IsEmptyPosition: Ticket ", OrderTicket(), " considered EMPTY POSITION");
		}
		else
			Print( "IsEmptyPosition: Ticket ", OrderTicket(), " at position ",
					iOrderPos, " is not ", Symbol(), " mgk:", iMagicNumber,
					" but ", OrderSymbol(), " mgk:", OrderMagicNumber(),
					" Type ", OrderType() );
	}

	return(true);
}
//+------------------------------------------------------------------+
bool IsBuyOrSellPosition(int & iTicket, int iMagicNumber, int iOrderType )
{
	if( iOrderType != OP_BUY && iOrderType != OP_SELL )
		return(false);

	for( int iOrderPos=OrdersTotal(); iOrderPos>0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos-1, SELECT_BY_POS ) )
		{
			Alert ( "IsBuyPosition: Unable to select order at position ",
					iOrderPos-1, " Error # ", GetLastError() );
			continue;
		}
		
		if( OrderType() == iOrderType &&
			OrderSymbol() == Symbol() &&
			OrderMagicNumber() == iMagicNumber )
		{
			iTicket = OrderTicket();
			
			return (true);
		}
	}

	return (false);
}
//+------------------------------------------------------------------+
bool IsBuyPosition(int & iTicket, int iMagicNumber)
{
	return( IsBuyOrSellPosition( iTicket, iMagicNumber, OP_BUY ) );
}
//+------------------------------------------------------------------+
bool IsSellPosition(int & iTicket, int iMagicNumber)
{
	return( IsBuyOrSellPosition( iTicket, iMagicNumber, OP_SELL ) );
}
//+------------------------------------------------------------------+
/*
int OpenAtMarket (  int iOrderType,
					double fLots,
					double fPrice,
					int iSlippage, 
					int iMagicNumber )
{
	if(iOrderType != OP_BUY && iOrderType != OP_SELL)
		return(-1);

// TODO : check for other limitations of OrderSend
// TODO : Implement context locking!
	if(fLots < 0.1)
		return(-1);

	string strOperation = OperationToStr( iOrderType );

	double fStopLoss = Ask-10000*Point;
	double fTakeProfit = Ask+999*Point;
	color iColor = RoyalBlue;
	
	if( OP_SELL == iOrderType )
	{		
		fStopLoss = Bid+10000*Point;
		fTakeProfit = Bid-999*Point;
		iColor = OrangeRed;
	}

	int iTicket = 
	OrderSend(	Symbol(), iOrderType, 
				NormalizeDouble(fLots, 1), NormalizeDouble(fPrice, 4),
				iSlippage, fStopLoss, fTakeProfit,
				strOperation, iMagicNumber, 0, iColor );

	if( -1 == iTicket )
	{
		Alert("OpenAtMarket: OrderSend Error ", GetLastError() );
	}
	
	return(iTicket);
}
//+------------------------------------------------------------------+
void CloseAllPositions( int iOrderType, 
						int iMagicNumber,
						double fPreferredClosePrice,
						int iSlippage, 
						color iColor )
{
	for( int iOrderPos=OrdersTotal(); iOrderPos>0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos-1, SELECT_BY_POS ) )
		{
			Alert ( "CloseAllPositions: Unable to select order at position ", 
					iOrderPos-1, " Error # ", GetLastError() );
		}
		else if(OrderSymbol() == Symbol() &&
				OrderMagicNumber() == iMagicNumber )
		{
			bool bCloseAll = (iOrderType==-1);

			if( bCloseAll || OrderType() == iOrderType )
			{
				int iTicket = OrderTicket();
				if( !OrderClose(iTicket, OrderLots(), fPreferredClosePrice, iSlippage, iColor) )
				{
					Alert ( "CloseAllPositions: Unable to Close Order #", iTicket,
							" at position ", iOrderPos-1,
							" Error # ", GetLastError() );
				}
			}
		}
	}
}
*/